#include "stdio.h"
#include "stdlib.h"
#include "mem.h"

#ifdef __MSDOS__
 #ifdef __BORLANDC__
  #define __DOS_GRAPHIC__
  #include "graphics.h"
  #include "conio.h"
 #endif
#endif

/*
Format of TI-92 PIC

Example of a 103 * 239 picture :

address   count   description
-------   -----   -----------
0000-0007     8 : "**TI92**" text
0008-0009     2 : 00 01 : seperator ???
000A-0011     8 : folder name
0012-0039    50 : "Bitmap file dated mm/dd/yy, hh:mm ^^^^^^^" text
003A-003B     2 : 0100 : seperator ???
003C-003F     4 : 52 00 00 00 : ???
0040-0047     8 : variable name
0048-004B     4 : 10 00 00 00 : ???
004C-004F     4 : file length
0050-0055     6 : A5 5A 00 00 00 00 : ???
= = = = = = = = = = = = = = = = = = = = = = = = = = = =
0056-0057     2 : size : 0C17 : 3095(d) bytes to read
0058-0059     2 : 103  : 0067 : 103(d) vertical resolution
005A-005B     2 : 239  : 00EF : 239(d) horizontal resolution
005C-0C6D  3090 : data : 103*240/8 = 3090 bytes stored in row major

  NOTE : the number of bits in each row will be aligned
         to a multiple of 8

0C6E-0C71     1 : DF : end mark
= = = = = = = = = = = = = = = = = = = = = = = = = = = =
0C6F-0C70     2 : checksum : from "bytes to read" to "end mark"
*/


void load_header_data(char * h) {
  FILE * f = fopen("header.dat", "rb");
  fread(h, 1, 86, f);
  fclose(f);
}

class picture {
    unsigned char header[86];
    unsigned int  bytes, v, h;
    unsigned char * data;
    unsigned char * gray;
    unsigned char end_mark;
    unsigned int checksum;
    unsigned int bpr; // bytes per row
  public:
    picture() {
      bytes = 0; v = 0; h = 0;
      data = NULL; gray = NULL;
      end_mark = 0xDF; checksum = 0;
      load_header_data(header);
    }
    ~picture() {
      if (!data) delete data;
      if (!gray) delete gray;
    }
    void set_xy(int x, int y);
    void load_raw_gray_image(char * filename);
    void convert();
    void set_checksum();
    void write_to_file(char * filename);
    void set_var(char * vn);
#ifdef __DOS_GRAPHIC__
    void show_raw_gray_image();
    void show_92i_image();
#endif
};

void picture::set_xy(int x, int y) {
  h = x;
  v = y;
  bpr = x / 8;
  if (x % 8) bpr++;
  data = new unsigned char[bpr * v];
  setmem(data, bpr * v, 0); // clear data
  bytes = bpr * v + 5; // 2 * 2 bytes for res. info + 1 byte for endmark
}

void picture::load_raw_gray_image(char * filename) {
  if (!h) {
    printf("[ ERROR ] : resolution not set\n");
    exit(-1);
  }

  FILE * f = fopen(filename, "rb");
  if (!f) {
    printf("[ ERROR ] : error opening raw image file\n");
    exit(-1);
  }

  gray = new unsigned char[v * h];
  if (fread(gray, 1, v * h, f) != v * h) {
    printf("[ ERROR ] : raw image file size not match\n");
    exit(-1);
  }
}

void picture::convert() {
 unsigned int x, y, p;
 char mask[8] = {128, 64, 32, 16, 8, 4, 2, 1};
 for(unsigned int i = 0; i < v * h; i++) { // total bytes in gray[]
    y = (v - 1) - (i % v);
    x = i / (v * 8);   // byte number
    p = (i / v) % 8;   // bit number
    if (gray[i]) data[y * bpr + x] |= mask[p];
  }
  for(i = 0; i < v * bpr; i++)
    data[i] = ~data[i];
}

void picture::set_checksum() {
  for(int i = 0; i < v * bpr; i++)
    checksum += (0x00ff & data[i]);
  checksum += ((0xff00 & bytes) >> 8);
  checksum += (0x00ff & bytes);
  checksum += ((0xff00 & v) >> 8);
  checksum += (0x00ff & v);
  checksum += ((0xff00 & h) >> 8);
  checksum += (0x00ff & h);
  checksum += (0x00ff & end_mark);
}

void picture::write_to_file(char * filename) {
  set_checksum();
  FILE * f = fopen(filename, "wb");
  // set file length in header
  unsigned long length = bpr * v + 95;
  header[0x004f] = (length & 0xff000000) >> 24;
  header[0x004e] = (length & 0x00ff0000) >> 16;
  header[0x004d] = (length & 0x0000ff00) >> 8;
  header[0x004c] = (length & 0x000000ff);

  unsigned char t;
  fwrite(&header, 86, 1, f);
  t = (bytes >> 8); fwrite(&t, 1, 1, f);
  t = (bytes & 0x00ff); fwrite(&t, 1, 1, f);
  t = (v >> 8); fwrite(&t, 1, 1, f);
  t = (v & 0x00ff); fwrite(&t, 1, 1, f);
  t = (h >> 8); fwrite(&t, 1, 1, f);
  t = (h & 0x00ff); fwrite(&t, 1, 1, f);
  fwrite(data, 1, bpr * v, f);
  fwrite(&end_mark, 1, 1, f);
  fwrite(&checksum, 2, 1, f);
  fclose(f);
}

void picture::set_var(char * vn) {
  int i = 0;
  while (vn[i] && (i < 8)) header[0x0040 + i] = vn[i++];
  while (i < 8) { header[0x0040 + i] = 0; i++; }
}

#ifdef __DOS_GRAPHIC__

void picture::show_raw_gray_image() {
  int x= 0, y = 20; // corner
  outtextxy(0, 0, "< raw gray image >");
  for(int i = 0; i < h; i++) {
    for(int j = 0; j < v; j++) {
      if (gray[i * v + j]) putpixel(x + j, y + i, 15);
      else putpixel(x + j, y + i, 1);
    }
  }
}

void picture::show_92i_image() {
  char mask[8] = {128, 64, 32, 16, 8, 4, 2, 1};
  int x= 240, y = 20; // corner
  outtextxy(240, 0, "< .92i image >");
  for(int i = 0; i < v; i++) {
    int c = 0;
    for(int j = 0; j < bpr; j++) {
      for(int k = 0; k < 8; k++) {
        if (data[i * bpr + j] & mask[k]) putpixel(x + c, y + i, 1);
        else putpixel(x + c, y + i, 15);
        c++;
      }
    }
  }
}

#endif

//--------------------------------------------------------------------

void main(int argc, char * argv[]) {
  printf("\n[ raw gray to .92i converter ] 0.2b / Roland Fox\n\n");

  char gray_fn[32], pic_fn[32], vn[9];
  int x, y, k;
  printf("1. raw gray filename : "); gets(gray_fn);
  printf("2.      .92 filename : "); gets(pic_fn);
  printf("3.     variable name : ( < 8 characters ) : "); gets(vn);
  printf("4.    pixels per row : ( y of raw image ) : "); scanf("%d", &x);
  printf("5.        total rows : ( x of raw image ) : "); scanf("%d", &y);
#ifdef __DOS_GRAPHIC__
  printf("6.  view pics. (1/0) : ( 1 -> YES ) : "); scanf("%d", &k);
#endif

  picture p;
  p.set_xy(x, y);
  p.load_raw_gray_image(gray_fn);
  p.convert();
  p.set_var(vn);
  p.write_to_file(pic_fn);

#ifdef __DOS_GRAPHIC__
  if (k > 0) {
    int gdriver = DETECT, gmode;
    initgraph(&gdriver, &gmode, "c:\\bc\\bgi");
    p.show_raw_gray_image();
    p.show_92i_image();
    while (!kbhit());
    closegraph();
  }
#endif
}